'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _lodashObjectAssign = require('lodash/object/assign');

var _lodashObjectAssign2 = _interopRequireDefault(_lodashObjectAssign);

var _lodashCollectionForEach = require('lodash/collection/forEach');

var _lodashCollectionForEach2 = _interopRequireDefault(_lodashCollectionForEach);

var _lodashObjectHas = require('lodash/object/has');

var _lodashObjectHas2 = _interopRequireDefault(_lodashObjectHas);

var _lodashLangIsEmpty = require('lodash/lang/isEmpty');

var _lodashLangIsEmpty2 = _interopRequireDefault(_lodashLangIsEmpty);

var _lodashObjectMapValues = require('lodash/object/mapValues');

var _lodashObjectMapValues2 = _interopRequireDefault(_lodashObjectMapValues);

var _lodashObjectOmit = require('lodash/object/omit');

var _lodashObjectOmit2 = _interopRequireDefault(_lodashObjectOmit);

var _lodashObjectPick = require('lodash/object/pick');

var _lodashObjectPick2 = _interopRequireDefault(_lodashObjectPick);

var _stampit = require('stampit');

var _stampit2 = _interopRequireDefault(_stampit);

var _utils = require('./utils');

var dupeFilter = function dupeFilter(prev, next, key, targ) {
  if (targ[key]) {
    throw new TypeError('Cannot mixin key `' + key + '` because it is provided by multiple sources.');
  }

  return next;
};

/**
 * React specification for creating new components
 */
var reactSpec = {
  propTypes: 'many_merged_dupe',
  defaultProps: 'many_merged_dupe',
  contextTypes: 'many_merged',
  childContextTypes: 'many_merged',
  getChildContext: 'many_merged_dupe',
  render: 'once',
  componentWillMount: 'many',
  componentDidMount: 'many',
  componentWillReceiveProps: 'many',
  shouldComponentUpdate: 'once',
  componentWillUpdate: 'many',
  componentDidUpdate: 'many',
  componentWillUnmount: 'many'
};

/**
 * Iterate through stamp methods, creating wrapper
 * functions for mixable React methods, starting
 * execution with first-in.
 *
 * @param {Object} targ Method destination
 * @param {Object} src New methods
 * @return {Object} An object of methods
 */
function wrapMethods(targ, src) {
  var methods = undefined;

  methods = (0, _lodashObjectMapValues2['default'])(src, function (val, key) {
    if (typeof val === 'function') {
      switch (reactSpec[key]) {
        case 'many':
          return function () {
            /* eslint-disable no-unused-expressions */
            targ[key] && targ[key].apply(this, arguments);
            val.apply(this, arguments);
            /* eslint-disable no-unused-expressions */
          };
        case 'many_merged_dupe':
          return function () {
            var res1 = targ[key] && targ[key].apply(this, arguments);
            var res2 = val.apply(this, arguments);

            return res1 ? (0, _lodashObjectAssign2['default'])(res1, res2, dupeFilter) : res2;
          };
        case 'once':
        default:
          if (!targ[key]) {
            return val;
          }

          throw new TypeError('Cannot mixin `' + key + '` because it has a unique constraint.');
      }
    }
  });

  return (0, _lodashObjectAssign2['default'])({}, targ, methods);
}

/**
 * Process the static properties of a stamp and
 * combine the result with the passed in statics object.
 *
 * @param {Object} stamp A stamp
 * @param {Object} prev An object of past static properties
 * @return {Object} A processed object of static properties
 */
function extractStatics(targ, src) {
  var statics = (0, _lodashObjectAssign2['default'])({}, targ);

  (0, _lodashCollectionForEach2['default'])(src, function (val, key) {
    if (reactSpec[key] === 'many_merged_dupe') {
      statics[key] = (0, _lodashObjectAssign2['default'])({}, statics[key], val, dupeFilter);
    } else if (reactSpec[key] === 'many_merged') {
      statics[key] = (0, _lodashObjectAssign2['default'])({}, statics[key], val);
    } else {
      statics[key] = val;
    }
  });

  return statics;
}

/**
 * Take two or more stamps produced from react-stampit or
 * stampit and combine them to produce and return a new stamp.
 *
 * @param {...Function} stamp Two or more stamps.
 * @return {Function} A new stamp composed from arguments.
 */
function compose() {
  for (var _len = arguments.length, stamps = Array(_len), _key = 0; _key < _len; _key++) {
    stamps[_key] = arguments[_key];
  }

  var result = (0, _stampit2['default'])(),
      refs = { state: {} },
      init = [],
      methods = {},
      statics = {};

  if ((0, _utils.isStamp)(this)) {
    stamps.push(this);
  }

  (0, _lodashCollectionForEach2['default'])(stamps, function (stamp) {
    stamp = !(0, _utils.isStamp)(stamp) ? rStampit(null, stamp) : stamp; // eslint-disable-line

    init = init.concat(stamp.fixed.init);
    methods = wrapMethods(methods, stamp.fixed.methods);
    statics = extractStatics(statics, (0, _lodashObjectOmit2['default'])(stamp, function (val, key) {
      return (0, _lodashObjectHas2['default'])(result, key);
    }));

    if (stamp.fixed.refs && stamp.fixed.refs.state) {
      (0, _lodashObjectAssign2['default'])(refs.state, stamp.fixed.refs.state, dupeFilter);
    }
  });

  result = result.init(init).refs(refs).methods(methods)['static'](statics);
  result.compose = compose;

  return (0, _utils.stripStamp)(result);
}

/**
 * Return a factory function (called a stamp) that will produce new
 * React components using the prototypes that are passed in or composed.
 *
 * @param {Object} React  The React library.
 * @param {Object} [props]  A map of property names and values specialized for React.
 * @return {Function} stamp A factory to produce React components using the given properties.
 * @return {Object} stamp.fixed An object map containing the fixed prototypes.
 */
function rStampit(React, props) {
  var stamp = React ? _stampit2['default'].convertConstructor(React.Component) : (0, _stampit2['default'])();
  var refs = undefined,
      methods = undefined,
      statics = undefined;

  // shortcut for converting React's class to a stamp
  if ((0, _lodashLangIsEmpty2['default'])(props)) {
    stamp.compose = compose;
    return (0, _utils.stripStamp)(stamp);
  }

  statics = (0, _lodashObjectAssign2['default'])({}, props.statics, (0, _lodashObjectPick2['default'])(props, ['contextTypes', 'childContextTypes', 'propTypes', 'defaultProps']));
  methods = (0, _lodashObjectOmit2['default'])(props, function (val, key) {
    return (0, _lodashObjectHas2['default'])(statics, key) || ['state', 'statics'].indexOf(key) >= 0;
  });
  refs = props.state && { state: props.state };

  stamp = stamp.refs(refs).methods(methods)['static'](statics);
  stamp.compose = compose;

  return (0, _utils.stripStamp)(stamp);
}

exports['default'] = (0, _lodashObjectAssign2['default'])(rStampit, { compose: compose, isStamp: _utils.isStamp });
exports.stamp = _utils.stamp;
exports.compose = compose;