'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.isStamp = isStamp;
exports.stripStamp = stripStamp;
exports.getNonEnum = getNonEnum;
exports.getEnum = getEnum;
exports.stamp = stamp;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _lodashObjectAssign = require('lodash/object/assign');

var _lodashObjectAssign2 = _interopRequireDefault(_lodashObjectAssign);

var _lodashObjectMerge = require('lodash/object/merge');

var _lodashObjectMerge2 = _interopRequireDefault(_lodashObjectMerge);

var _stampit = require('stampit');

var _stampit2 = _interopRequireDefault(_stampit);

var _stampit3 = require('./stampit');

function isStamp(obj) {
  return typeof obj === 'function' && typeof obj.compose === 'function' && typeof obj.fixed === 'object';
}

function stripStamp(stamp) {
  delete stamp.create;
  delete stamp.init;
  delete stamp.methods;
  delete stamp.state;
  delete stamp.refs;
  delete stamp.props;
  delete stamp.enclose;
  delete stamp['static'];

  return stamp;
}

/**
 * Get object of non-enum properties
 */

function getNonEnum(target) {
  var props = Object.getOwnPropertyNames(target);
  var enumOnly = Object.keys(target);
  var obj = {};

  props.forEach(function (key) {
    var indexInEnum = enumOnly.indexOf(key);
    if (indexInEnum === -1 && key !== 'constructor') {
      obj[key] = target[key];
    }
  });

  return obj;
}

/**
 * Get object of enum properties
 */

function getEnum(target) {
  var props = Object.keys(target);
  var obj = {};

  props.forEach(function (key) {
    obj[key] = target[key];
  });

  return obj;
}

/**
 * ES7 decorator for converting ES6 class to stamp
 */

function stamp(Class) {
  var constructor = function constructor() {
    (0, _lodashObjectMerge2['default'])(this, new Class());
  };
  var methods = (0, _lodashObjectAssign2['default'])({}, Object.getPrototypeOf(Class).prototype, getNonEnum(Class.prototype));

  var stamp = _stampit2['default'].init(constructor).methods(methods)['static'](getEnum(Class));
  stamp.compose = _stampit3.compose;

  return stripStamp(stamp);
}